#ifndef __DJ_SOCK_PRIVATE__
#define __DJ_SOCK_PRIVATE__ 1
#define __dj_ENFORCE_FUNCTION_CALLS 1
#define __NO_INLINE__ 1
#include <tcp.h>
#undef __dj_ENFORCE_FUNCTION_CALLS
#undef __NO_INLINE__
typedef unsigned char   BYTE;
typedef unsigned short  WORD;
typedef unsigned long   DWORD;
typedef unsigned int    UINT;
#define UDP_TCP_COMMON
#define BOOL int
#define udp_MaxBufSize    1520
#define tcp_MaxBufSize    2048
#define tcp_MaxTxBufSize  tcp_MaxBufSize
#define MAX_FRAGMENTS   30UL
#define MAX_WINDOW      (32*1024U)
#define ETH_MAX_DATA     1500
#define SOMAXCONN       32

/*!\struct in_Header
 *
 * The Internet (ip) Header.
 */
typedef struct in_Header {
#if defined(USE_BIGENDIAN)
        BYTE   ver    : 4;
        BYTE   hdrlen : 4;
#elif defined(OLD_TURBOC) || defined(__CCDL__)
        int    hdrlen : 4;
        int    ver    : 4;
#else
        BYTE   hdrlen : 4;     /* Watcom requires BYTE here */
        BYTE   ver    : 4;
#endif
        BYTE   tos;
        WORD   length;
        WORD   identification;
        WORD   frag_ofs;
        BYTE   ttl;
        BYTE   proto;
        WORD   checksum;
        DWORD  source;
        DWORD  destination;
      } in_Header;

/*!\struct recv_data
 */
typedef struct recv_data {
        DWORD  recv_sig;
        BYTE  *recv_bufs;
        WORD   recv_bufnum;
      } recv_data;

/*!\struct recv_buf
 */
typedef struct recv_buf {
        DWORD       buf_sig;
        DWORD       buf_hisip;
        long        buf_seqnum;
#if defined(USE_IPV6)
        ip6_address buf_hisip6;
#endif
        WORD        buf_hisport;
        short       buf_len;
        BYTE        buf_data [ETH_MAX_DATA];
      } recv_buf;


#define MAX_IP4_DATA      (ETH_MAX_DATA - sizeof(in_Header))
#define MAX_FRAG_SIZE     (MAX_FRAGMENTS * MAX_IP4_DATA)

/*!\struct in6_Header
 *
 * IPv6 header.
 */
typedef struct in6_Header {
#if defined(USE_BIGENDIAN)
        BYTE        ver : 4;
        BYTE        pri : 4;
#else
        BYTE        pri : 4;
        BYTE        ver : 4;
#endif
        BYTE        flow_lbl[3];
        WORD        len;
        BYTE        next_hdr;
        BYTE        hop_limit;
        ip6_address source;
        ip6_address destination;
      } in6_Header;

#define MAX_IP6_DATA      (ETH_MAX_DATA - sizeof(in6_Header))

recv_buf          **packet_pool;


/*!\struct _udp_Socket
 *
 * UDP socket definition.
 */
typedef struct _udp_Socket {
        struct _udp_Socket *next;
        UDP_TCP_COMMON;
        BYTE rx_buf[udp_MaxBufSize+1]; /**< received data buffer */

#if defined(USE_IPV6)
        ip6_address  my6addr;          /**< my ip6-address */
        ip6_address  his6addr;         /**< peer's ip-6 address */
#endif
        DWORD        safetysig;        /**< magic marker */
      } _udp_Socket;


typedef struct _tcp_Socket {
        struct _tcp_Socket *next;      /**< link to next tcp-socket */
        UDP_TCP_COMMON;

        BYTE rx_buf[tcp_MaxBufSize+1]; /**< received data buffer */

#if defined(USE_IPV6)
        ip6_address  my6addr;          /**< our IPv6 address */
        ip6_address  his6addr;         /**< peer's IPv6 address */
#endif
        UINT         state;            /**< tcp connection state */
        DWORD        recv_next;        /**< SEQ number we expect to receive */
        DWORD        send_next;        /**< SEQ we send but not ACK-ed by peer */
        long         send_una;         /**< unacked send data, must be signed */

#if defined(USE_DEBUG)
        DWORD        last_acknum[2];   /**< for pcdbug.c; to follow SEQ/ACK */
        DWORD        last_seqnum[2];   /**< increments */
#endif
        DWORD        timeout;          /**< timer for retrans etc. */
        BYTE         unhappy;          /**< flag, indicates retransmitting segt's */
        BYTE         recent;           /**< 1 if recently transmitted */
        WORD         flags;            /**< TCP flags used in next Tx */

        UINT         window;           /**< other guy's window */
        UINT         adv_win;          /**< our last advertised window */

        BYTE         cwindow;          /**< Congestion window */
        BYTE         wwindow;          /**< Van Jacobson's algorithm */
        WORD         fill_4;

        DWORD        vj_sa;            /**< VJ's alg, standard average   (SRTT) */
        DWORD        vj_sd;            /**< VJ's alg, standard deviation (RTTVAR) */
        DWORD        vj_last;          /**< last transmit time */
        UINT         rto;              /**< retransmission timeout */
        BYTE         karn_count;       /**< count of packets */
        BYTE         tos;              /**< TOS from IP-header */
        WORD         fill_5;

        DWORD        rtt_time;         /**< Round Trip Time value */
        DWORD        rtt_lasttran;     /**< RTT at last transmission */

        DWORD        ts_sent;          /**< last TimeStamp value sent */
        DWORD        ts_recent;        /**< last TimeStamp value received */
        DWORD        ts_echo;          /**< last TimeStamp echo received */

        UINT         max_seg;          /**< MSS for this connection */

        /** S. Lawson - handle one dropped segment.
         * \todo Make a prober re-assembly queue.
         * missed_seq[0] is left edge of missing segment.
         * missed_seq[1] is right edge (in peer's absolute SEQ space)
         */
        DWORD        missed_seq[2];
     /* void        *reasm_buf; */     /**< linked-list of frags; not yet */

#if defined(USE_TCP_MD5)
        char        *secret;           /**< Secret for MD5 finger-print */
#endif
        DWORD        inactive_to;      /**< inactive timer (no Rx data) */
        DWORD        datatimer;        /**< inactive timer (no Tx data) */
     /* int          sock_delay; ?? */

        BYTE         tx_wscale;        /**< \todo window scales shifts, Tx/Rx */
        BYTE         rx_wscale;
        UINT         tx_queuelen;      /**< optional Tx queue length */
        const BYTE  *tx_queue;

        UINT         tx_datalen;       /**< number of bytes of data to send */
        UINT         max_tx_data;      /**< Last index for tx_data[] */
        BYTE        *tx_data;          /**< Tx data buffer (default tx_buf[]) */
        BYTE         tx_buf[tcp_MaxTxBufSize+1]; /**< data for transmission */
        DWORD        safetysig;        /**< magic marker */
        DWORD        safetytcp;        /**< extra magic marker */
      } _tcp_Socket;


typedef struct _raw_Socket {
        struct _raw_Socket *next;
        WORD   ip_type;                  /**< same ofs as for udp/tcp Socket */
        BOOL   used;                     /**< used flag; packet not read yet */
        DWORD  seq_num;                  /**< counter for finding oldest pkt */
        struct in_Header ip;
        BYTE   rx_data [MAX_FRAG_SIZE];  /**< room for 1 jumbo IP packet */
} _raw_Socket;

typedef struct _raw6_Socket {
        struct _raw6_Socket *next;
        WORD   ip_type;                  /**< same ofs as for udp/tcp Socket */
        BOOL   used;                     /**< used flag; packet not read yet */
        DWORD  seq_num;                  /**< counter for finding oldest pkt */
        struct in6_Header ip6;
        BYTE   rx_data [MAX_IP6_DATA];  /**< room for 1 jumbo IP packet */
} _raw6_Socket;


typedef struct Socket {
        int                 fd;
        struct Socket      *next;
        struct sockaddr_in *local_addr;
        struct sockaddr_in *remote_addr;

        int                 timeout;
        int                 inp_flags;   /* misc. IP-protocol options */
        BYTE                ip_tos;

        BYTE                ip_ttl;
        struct ip_opts     *ip_opt;      /* for setsockopt(s,IP_OPTION,..) */
        unsigned            ip_opt_len;  /* output IP-option length */
        int                 so_options;  /* SO_KEEPALIVE, SO_ACCEPTCONN etc. */
        int                 so_proto;    /* IPPROTO_UDP, IPPROTO_TCP etc. */
        int                 so_type;     /* SOCK_STREAM, SOCK_DGRAM etc. */
        int                 so_family;   /* address family AF_INET/AF_INET6 */
        int                 so_state;    /* internal state, ref SS_* below */
        int                 so_error;    /* internal socket errno */
        time_t              close_time;  /* we closed at (SOCK_STREAM) */
        BOOL                linger_on;   /* user specified lingering */
        unsigned            linger_time; /* linger-time (SOCK_STREAM) */
        unsigned            fd_duped;    /* FSEXT reference counting */
        DWORD               keepalive;   /* keepalive timeout */
        DWORD               nb_timer;    /* non-block timer */
        _udp_Socket        *udp_sock;    /* actual state and Rx/Tx data is in */
        _tcp_Socket        *tcp_sock;    /*  one of these pointers */
        _raw_Socket        *raw_sock;    /* points to 1st in linked-list */
        _raw6_Socket       *raw6_sock;
        recv_buf          **bcast_pool;  /* buffers for INADDR_ANY sockets */
        unsigned            pool_size;   /* size of above buffer */

        /* For SOCK_PACKET sockets.
        */
        recv_buf          **packet_pool;
        int               (*old_eth_peek) (void*);

        /* listen-queue for incoming tcp connections
         */
        int                 backlog;
        _tcp_Socket        *listen_queue [SOMAXCONN];
        DWORD               syn_timestamp[SOMAXCONN]; /* got SYN at [msec] */
        unsigned            send_lowat;  /* low-water Tx marks */
        unsigned            recv_lowat;  /* low-water Rx marks */
        DWORD               cookie;      /* memory cookie / marker */

 } Socket;
#endif
